#include <stdio.h>
#include <stdlib.h>
#include <ScreenSaver.h>
#include <Screen.h>
#include "ConfigView.h"

struct WakeLine
{
	int		x1;					// x-coordinate on screen of point 1
	int		y1;					// y-coordinate on screen of point 1
	int		x2;					// x-coordinate on screen of point 2
	int     y2;					// y-coordinate on screen of point 2
	int		red;				// Line colors
	int		green;
	int		blue;
};

const int			nLines = 50;		// Num of Lines of the Wake
int					dx1;				// Increment on x axis of point 1
int					dy1;				// Increment on y axis of point 1
int					dx2;				// Increment on x axis of point 2
int					dy2;				// Increment on y axis of point 2
WakeLine     		myWake[nLines];
bool				dxy1changed;
bool				dxy2changed;

void InitWake(void)				// initialize Wake
{
	myWake[0].x1	= 50;
	myWake[0].y1	= 30;
	myWake[0].x2    = 75;
	myWake[0].y2    = 100;
	myWake[0].red	= 25;
	myWake[0].green = 15;
	myWake[0].blue  = 256; 
	
	dx1 = 3;
	dy1 = 3;
	dx2 = 4;
	dy2 = 4;
	
	for (int i=1; i<nLines; i++)
	{
		myWake[i].red	= 25;
		myWake[i].green = 15;
		myWake[i].blue  = 256 - (200 / nLines) * (i+1);
	}
	
	dxy1changed = false;
	dxy2changed = false;
}

class BeWake : public BScreenSaver
{

public:
				BeWake(BMessage *message, image_id image);
				~BeWake();
	void		StartConfig(BView *view);
	status_t 	SaveState(BMessage *into) const;
	status_t	StartSaver(BView *v, bool preview);
	void        StopSaver();
	void 		StopConfig();
	void		Draw(BView *v, int32 frame);
	
    ConfigView *configs;
  	
  	bool        BlackBackground;
  	bool		blWaitForRetrace;
};

// MAIN INSTANTIATION FUNCTION
extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, image_id image)
{
	return new BeWake(message, image);
}

BeWake::BeWake(BMessage *message, image_id image)
 : BScreenSaver(message, image)
{
  if(message->FindBool("BlackBackground", &BlackBackground) != B_OK)
  {
    BlackBackground = true;
  }
  
  if(message->FindBool("WaitForRetrace", &blWaitForRetrace) != B_OK)
  {
    blWaitForRetrace = false;
  }
  
}

// Saves the state of the screensaver (stores config variables)
status_t BeWake::SaveState(BMessage *into) const
{
  into->AddBool("BlackBackground", BlackBackground);
  into->AddBool("WaitForRetrace", blWaitForRetrace);
  
  return B_OK;
}



//********************************************************
//	The BView shown in the screensaver-preference window
//********************************************************
void BeWake::StartConfig(BView *view)
{	
	
	configs = new ConfigView(view->Bounds(), "configs", &BlackBackground, &blWaitForRetrace);
	view->AddChild(configs);
	
}

//********************************************************
//	initialize screensaver
//********************************************************
status_t BeWake::StartSaver(BView *view, bool preview)
{
  	
  	InitWake();
  	
  	SetTickSize(20000);
  	
  	return B_OK;
}

void BeWake::StopSaver(void)
{
}

void BeWake::StopConfig(void)
{
	
}

BeWake::~BeWake(void)
{
}

//********************************************************
//	update the Wake-Movement
//********************************************************
void MoveLines(BView *view)
{
		
	int		x1, y1, x2, y2;

	x1 = myWake[0].x1;
	y1 = myWake[0].y1;
	x2 = myWake[0].x2;
	y2 = myWake[0].y2;
					
	BRect	r=view->Bounds();
		
	x1 = x1 + dx1;
	y1 = y1 + dy1;
		
	if ((x1 > r.right) || (x1 < r.left))
	{
	   dx1 = -1 * dx1;
	   x1 = x1 + 2 * dx1;
	}
		
	if ((y1 > r.bottom) || (y1 < r.top))
	{
	   dy1 = -1 * dy1;
	   y1 = y1 + 2 * dy1;
	}
		
	x2 = x2 + dx2;
	y2 = y2 + dy2;
		
	if ((x2 > r.right) || (x2 < r.left))
	{
	   dx2 = -1 * dx2;
	   x2 = x2 + 2 * dx2;
	}
		
	if ((y2 > r.bottom) || (y2 < r.top))
	{
	   dy2 = -1 * dy2;
	   y2 = y2 + 2 * dy2;
	}
		
	for (int i= (nLines - 1); i > 0 ; i--)
	{
		myWake[i].x1 = myWake[i-1].x1;
		myWake[i].y1 = myWake[i-1].y1;
		myWake[i].x2 = myWake[i-1].x2;
		myWake[i].y2 = myWake[i-1].y2;
		//myWake[i].red = myWake[i-1].red;
		//myWake[i].green = myWake[i-1].green;
		//myWake[i].blue = myWake[i-1].blue;
	}
	
	myWake[0].x1 = x1;
	myWake[0].y1 = y1;
	myWake[0].x2 = x2;
	myWake[0].y2 = y2;
	
}

void DrawLines(BView *view, bool erase)
{
		BPoint	p1, p2;
		
		for (int i=nLines-1;i>= 0;i--) {
			
		  p1.x = myWake[i].x1;
		  p1.y = myWake[i].y1;
		  p2.x = myWake[i].x2;
		  p2.y = myWake[i].y2;
		  
		  if (erase) 
		  {
		  	view->SetHighColor(0, 0, 0);
		  }
		  else
		  {
		  	view->SetHighColor(myWake[i].red, myWake[i].green, myWake[i].blue);
		  }
		  
		  view->StrokeLine(p1, p2, B_SOLID_HIGH);
		  
		}
}

void EraseLastLine(BView *view)
{
		BPoint	p1, p2;
		
		p1.x = myWake[nLines-1].x1;
		p1.y = myWake[nLines-1].y1;
		p2.x = myWake[nLines-1].x2;
		p2.y = myWake[nLines-1].y2;
		  
		view->SetHighColor(0, 0, 0);
		
		view->StrokeLine(p1, p2, B_SOLID_HIGH);

}

//********************************************************
//	draw all wake's lines to the screen
//********************************************************
void BeWake::Draw(BView *view, int32 frame)
{
	if ((frame ==0) && (BlackBackground))
	{
		view->SetHighColor(0,0,0);
		view->FillRect(view->Bounds());
	}
	
    // Used to sync to vertical retrace; this makes for a smoother display on some systems, as
    //  drawing will take place during the vertical blank -- and less flickering will be evident
    if (blWaitForRetrace)
    {
    	BScreen screen;
    	screen.WaitForRetrace();
	}
	
	if ((frame % 1000) == 0)
	{
		if (dxy1changed)
		{
			dx1 = 3;
			dy1 = 3;
		}
		else
		{
			dx1 = dx1 + rand() % 3;
			dy1 = dy1 + rand() % 3;
		}
	}
	
	if ((frame % 750) == 0)
	{
		if (dxy2changed)
		{
			dx2 = 4;
			dy2 = 4;
		}
		else
		{
			dx2 = dx2 + rand() % 3;
			dy2 = dy2 + rand() % 3;
		}
	}
	
	//DrawLines(view, true);
	EraseLastLine(view);
	
	MoveLines(view);
	
	DrawLines(view, false);
	
}
